import React, { useState, useEffect, useContext } from "react";
import Context from "./index";
import { isBusiness } from "../../components/constant/menu";
import { UserContext } from "../loginContext/loginContext";
import { notification } from "antd";
import { useRouter } from "next/router";

function calculateDiscountPercentage(originalPrice, discountedPrice) {
  if (
    originalPrice <= 0 ||
    (discountedPrice !== undefined &&
      (discountedPrice <= 0 || discountedPrice >= originalPrice))
  ) {
    // Invalid input, handle accordingly (return an error or handle as needed)
    return null;
  }

  // Check if discounted price is not given or is 0
  if (discountedPrice === undefined || discountedPrice === 0) {
    return 0;
  }

  // Check if prices are the same
  if (originalPrice === discountedPrice) {
    return 0;
  }

  // Calculate the discount percentage
  const discount = originalPrice - discountedPrice;
  const discountPercentage = (discount / originalPrice) * 100;

  // Round to two decimal places
  return Math.round(discountPercentage * 100) / 100;
}

const getLocalCartItems = () => {
  try {
    const list = localStorage.getItem("cartList");
    if (list === null) {
      return [];
    } else {
      return JSON.parse(list);
    }
  } catch (err) {
    return [];
  }
};

const CartProvider = (props) => {
  const { user } = useContext(UserContext);
  const [cartItems, setCartItems] = useState(getLocalCartItems());
  const [cartTotal, setCartTotal] = useState(0);
  const [quantity, setQuantity] = useState();
  const [stock, setStock] = useState("In Stock");

  useEffect(() => {
    const Total = cartItems.reduce((a, b) => a + b.total, 0);
    setCartTotal(Total);

    localStorage.setItem("cartList", JSON.stringify(cartItems));
  }, [cartItems]);

  // Add Product To Cart
  const addToCart = (item, quantity) => {
    const index = cartItems.findIndex((itm) => itm.id === item.id);
    const disc = calculateDiscountPercentage(
      item?.price,
      isBusiness(user) ? item?.wholesale_price : item?.discounted_price
    );
    if (index !== -1) {
      let oldItem = cartItems[index];
      cartItems[index] = {
        ...item,
        ship_cost: 0,
        qty: quantity + oldItem.qty,
        total:
          (isBusiness(user)
            ? item?.wholesale_price
            : item.discounted_price || item.price) *
          (quantity + oldItem.qty),
        price: isBusiness(user)
          ? item?.wholesale_price
          : item.discounted_price || item.price,
      };
      setCartItems([...cartItems]);
      notification.info({
        message: `Product Updated`,
        description: "Product Quantity Updated Successfully!",
        placement: "bottomRight",
      });
    } else {
      const product = {
        ...item,
        qty: quantity,
        ship_cost: 0,
        total:
          (isBusiness(user)
            ? item?.wholesale_price
            : disc
            ? item.discounted_price
            : item.price) * quantity,
        price: isBusiness(user)
          ? item?.wholesale_price
          : disc
          ? item.discounted_price
          : item.price,
      };
      setCartItems([...cartItems, product]);
      notification.success({
        message: `Product Added`,
        description: "Product added successfully in cart!",
        placement: "bottomRight",
      });
    }
    // router.push(`/page/account/cart`);
  };

  const removeFromCart = (item) => {
    notification.success({
      message: `Product Removed`,
      description: "Product removed Successfully from cart!",
      placement: "bottomRight",
    });
    setCartItems(cartItems.filter((e) => e.id !== item.id));
  };

  const minusQty = () => {
    if (quantity > 1) {
      setQuantity(quantity - 1);
      setStock("In Stock");
    }
  };

  const plusQty = (item) => {
    // if (item.stock_quantity >= quantity) {
    if (quantity) {
      setQuantity(quantity + 1);
    } else {
      setStock("Out of Stock !");
    }
  };

  const updateQty = (item, quantity) => {
    const disc = calculateDiscountPercentage(
      item?.price,
      isBusiness(user) ? item?.wholesale_price : item?.discounted_price
    );
    if (
      quantity >= (isBusiness(user) ? item.minimum_quantity : 1) &&
      item.available_stock >= quantity
    ) {
      const index = cartItems.findIndex((itm) => itm.id === item.id);
      if (index !== -1) {
        cartItems[index] = {
          ...item,
          qty: quantity,
          ship_cost: 0,
          total:
            (isBusiness(user)
              ? item?.wholesale_price
              : disc
              ? item.discounted_price
              : item.price) * quantity,
          price: isBusiness(user)
            ? item?.wholesale_price
            : disc
            ? item.discounted_price
            : item.price,
        };
        setCartItems([...cartItems]);
      } else {
        const product = {
          ...item,
          qty: quantity,
          ship_cost: 0,
          total:
            (isBusiness(user)
              ? item?.wholesale_price
              : disc
              ? item.discounted_price
              : item.price) * quantity,
          price: isBusiness(user)
            ? item?.wholesale_price
            : disc
            ? item.discounted_price
            : item.price,
        };
        setCartItems([...cartItems, product]);
        notification.success({
          message: `Product Added`,
          description: "Product added successfully in cart!",
          placement: "bottomRight",
        });
      }
    } else {
      if (item.available_stock >= quantity)
        notification.error({
          message: `Out of Stock`,
          description: "Product Quantity Out of Stock!",
          placement: "bottomRight",
        });
    }
  };

  return (
    <Context.Provider
      value={{
        ...props,
        state: cartItems,
        cartTotal,
        setQuantity,
        quantity,
        stock,
        addToCart: addToCart,
        // addQty: addQty,
        removeFromCart: removeFromCart,
        setCartItems: setCartItems,
        setCartTotal: setCartTotal,
        plusQty: plusQty,
        minusQty: minusQty,
        updateQty: updateQty,
      }}
    >
      {props.children}
    </Context.Provider>
  );
};

export default CartProvider;
