const fs = require("fs");
const path = require("path");

const saveErrorLog = (error) => {
  const logFilePath = path.join(__dirname, "RunTimeError.log");

  const logMessage = `
  Date: ${new Date().toISOString()}
  Error: ${error.name}
  Message: ${error.message}
  Stack Trace:
  ${error.stack}
  -----------------------------------------
  `;

  fs.mkdir(path.dirname(logFilePath), { recursive: true }, (err) => {
    if (err) {
      console.error("Error creating log directory:", err);
      return;
    }

    fs.appendFile(logFilePath, logMessage, (appendErr) => {
      if (appendErr) {
        console.error("Error saving error log:", appendErr);
      }
    });
  });
};

exports.catchErrors = (fn) => {
  return function (req, res, next) {
    const resp = fn(req, res, next);
    if (resp instanceof Promise) {
      return resp.catch(next);
    }
    return resp;
  };
};

exports.handleCatchError = (error) => {
  console.log(error);
  let statusCode = 500;
  let errorMessage = "An error occurred";
  saveErrorLog(error);
  if (error.name === "ValidationError") {
    statusCode = 400;
    const validationErrors = Object.values(error.errors).map(
      (err) => err.message
    );
    errorMessage = `Validation failed. Please check the following fields: ${validationErrors.join(
      ", "
    )}`;
  } else if (error.name === "MongoError" && error.code === 11000) {
    statusCode = 400;
    const duplicateFields = Object.keys(error.keyValue).join(", ");
    errorMessage = `Duplicate values found for unique fields: ${duplicateFields}`;
  } else if (
    error instanceof SyntaxError &&
    error.status === 400 &&
    "body" in error
  ) {
    statusCode = 400;
    errorMessage = "Bad Request. Invalid JSON format in request body.";
  } else if (error instanceof ReferenceError) {
    statusCode = 500;
    errorMessage = `Reference error: ${error.message}`;
  } else if (error instanceof TypeError) {
    statusCode = 500;
    errorMessage = `Type error: ${error.message}`;
  }
  // Add more conditions for other error types if needed

  return { statusCode, errorMessage };
};

// Usage example
try {
  // Code that might throw an error
} catch (error) {
  saveErrorLog(error);
}

exports.Handler = (err, req, res, next) => {
  const statusCode = err.status || 500;
  const errorMessage = err.message;
  console.error(err);

  res.status(statusCode).json({
    success: false,
    message: errorMessage,
  });
};
